@@ -24,7 +24,6 @@ import datetime
 import sqlalchemy
 
 import numpy as np
-from itertools import groupby, count
 
 import warnings
 
@@ -105,20 +104,24 @@ class SimpleTree:
                 only_list.append(esgf_query.descend_tree(self.pointers,self.drs,self.header,search_path,options,list_level=options.list_only_field))
             else:
                 esgf_query.descend_tree(self.pointers,self.drs,self.header,search_path,options)
+        #print json.dumps(self.pointers.tree,sort_keys=True, indent=4)
 
         if options.list_only_field!=None:
             for field_name in set([item for sublist in only_list for item in sublist]):
                 print field_name
         else:
             valid_experiments_path.intersection(self,self.drs)
+            #print json.dumps(self.pointers.tree,sort_keys=True, indent=4)
             #List domains:
             self.header['domain_list']=self.list_domains()
             self.create_netcdf_container(options,'record_paths')
         return
 
     def simulations_list(self):
-        simulations_list=self.list_subset((File_Expt.institute,File_Expt.model,File_Expt.ensemble))
-        return [simulation for simulation in simulations_list if simulation[2]!='r0i0p0']
+        ensemble_index=self.drs.simulations_desc.index('ensemble')
+        subset_desc=(getattr(File_Expt,item) for item in self.drs.simulations_desc)
+        simulations_list=self.list_subset(subset_desc)
+        return [simulation for simulation in simulations_list if simulation[ensemble_index]!='r0i0p0']
 
     def simulations(self,options):
         self.pointers.slice(options)
@@ -141,18 +144,18 @@ class SimpleTree:
                          )
                       )
 
-    def list_paths(self,options):
+    def get_paths_list(self,options):
+        self.pointers.create_database(find_simple)
+        subset=tuple([File_Expt.path]+[getattr(File_Expt,item) for item in self.drs.official_drs])
+        return sorted(list(set(self.list_subset(subset))))
+
+    def list_fields(self,options):
         #slice with options:
         self.pointers.slice(options)
         self.pointers.create_database(find_simple)
-        subset=tuple([File_Expt.path]+[getattr(File_Expt,item) for item in self.drs.official_drs])
-        paths_list=sorted(list(set(self.list_subset(subset))))
-        #paths_list=sorted(list(set([path[0] for path in self.list_subset((File_Expt.path,))])))
-        for path in paths_list:
-            if 'download' in dir(options) and options.download!='':
-                retrieve_path(options,path)
-            else:
-                print path[0]
+        fields_list=sorted(list(set(self.list_subset((getattr(File_Expt,field) for field in options.field)))))
+        for field in fields_list:
+            print ','.join(field)
         return
 
     def optimset(self,options):
@@ -161,6 +164,7 @@ class SimpleTree:
 
         #Find the list of institute / model with all the months for all the years / experiments and variables requested:
         valid_experiments_time.intersection(self,self.drs)
+        #print json.dumps(self.pointers.tree,sort_keys=True, indent=4)
         
         self.create_netcdf_container(options,'record_meta_data')
         return
@@ -219,78 +223,6 @@ class SimpleTree:
         self.create_netcdf_container(options,'record_paths')
         return
 
-#    def find_local(self,options):
-#        import database_utils
-#        self.pointers.slice(options)
-#
-#        #In the search paths, find which one are local:
-#        local_search_path=[ top_path for top_path in 
-#                                [os.path.expanduser(os.path.expandvars(search_path)) for search_path in self.header['search_list']]
-#                                if os.path.exists(top_path)]
-#        local_search_path=[ os.path.abspath(top_path) for top_path in local_search_path]
-#
-#        #For each remote file type, find the paths and see if a local copy exists:
-#        path_equivalence=dict()
-#        options.file_type=['HTTPServer','GridFTP']
-#        pointers_copy=copy.deepcopy(self.pointers.tree)
-#        self.pointers.slice(options)
-#
-#        #for path in sorted(list(set(self.pointers.level_list_last()))):
-#        self.pointers.create_database(find_simple)
-#        paths_list=sorted(list(set([path[0] for path in self.list_subset((File_Expt.path,))])))
-#
-#        for path in paths_list:
-#            for search_path in local_search_path:
-#                md5checksum=path.split('|')[1]
-#                path_to_file=search_path+'/'+'/'.join(path.split('|')[0].split('/')[-10:])
-#                try:
-#                    f=open(path_to_file)
-#                    if netcdf_utils.md5_for_file(f)==md5checksum:
-#                        path_equivalence[path]=path_to_file
-#                        break
-#                except:
-#                    pass
-#
-#        #Replace original tree:
-#        options.file_type=None
-#        self.pointers.tree=pointers_copy
-#
-#        self.pointers.replace_last(path_equivalence)
-#        return
-
-def retrieve_path(options,path):
-    decomposition=path[0].split('|')
-    if not (isinstance(decomposition,list) and len(decomposition)>1):
-        return
-
-    root_path=decomposition[0]
-    dest_name=options.download+'/'.join(path[1:])+'/'+root_path.split('/')[-1]
-    print dest_name
-    print root_path
-    try:
-        md5sum=netcdf_utils.md5_for_file(open(dest_name,'r'))
-    except:
-        md5sum=''
-    if md5sum==decomposition[1]:
-        print 'File '+dest_name+' found.'
-        print 'MD5 OK! Not retrieving.'
-        return
-
-    retrieval_utils.download_secure(root_path,dest_name)
-    try:
-        md5sum=netcdf_utils.md5_for_file(open(dest_name,'r'))
-    except:
-        md5sum=''
-    print 'Checking MD5 checksum of retrieved file...'
-    if md5sum!=decomposition[1]:
-        print('File '+dest_name+' does not have the same MD5 checksum as published on the ESGF. Removing this file...')
-        try:
-            os.remove(dest_name)
-        except:
-            pass
-    else:
-        print 'MD5 OK!'
-    return
 
 #def remote_retrieve(options):
 #    data=netCDF4.Dataset(options.in_diagnostic_netcdf_file,'r')
@@ -320,25 +252,22 @@ def retrieve_path(options,path):
 #
 #    return
 
-def convert_indices_to_slices(indices):
-    return [[next(v)] + list(v)[-1:] for k,v in groupby(indices, lambda x,c=count(): x-next(c))]
-
-def organize_identical_paths(paths_list):
-    #FIND UNIQUE PATHS AND ENSURE THAT THEY ARE SORTED:
-    sort_timestamp=np.argsort([item['timestamp'] for item in paths_list])
-    unique_paths, unique_indices=np.unique(np.array([item['path'] for item in paths_list])[sort_timestamp],return_index=True)
-    unique_paths=np.array([item['path'] for item in paths_list])[sort_timestamp][unique_indices]
-    paths_organized={}
-    for path in unique_paths:
-        indices=[item['index'] for item in paths_list if item['path']==path]
-        indices_sort=np.argsort(indices)
-        if len(indices)>1: indices=np.array(indices)[indices_sort]
-        paths_organized[path]={
-                                'indices': indices,
-                                'timestamps':np.array([item['timestamp'] for item in paths_list if item['path']==path])[indices_sort],
-                                'domains':[item['domains'] for item in paths_list if item['path']==path][0]
-                              }
-    return paths_organized, unique_paths
+#def organize_identical_paths(paths_list):
+#    #FIND UNIQUE PATHS AND ENSURE THAT THEY ARE SORTED:
+#    sort_timestamp=np.argsort([item['timestamp'] for item in paths_list])
+#    unique_paths, unique_indices=np.unique(np.array([item['path'] for item in paths_list])[sort_timestamp],return_index=True)
+#    unique_paths=np.array([item['path'] for item in paths_list])[sort_timestamp][unique_indices]
+#    paths_organized={}
+#    for path in unique_paths:
+#        indices=[item['index'] for item in paths_list if item['path']==path]
+#        indices_sort=np.argsort(indices)
+#        if len(indices)>1: indices=np.array(indices)[indices_sort]
+#        paths_organized[path]={
+#                                'indices': indices,
+#                                'timestamps':np.array([item['timestamp'] for item in paths_list if item['path']==path])[indices_sort],
+#                                'domains':[item['domains'] for item in paths_list if item['path']==path][0]
+#                              }
+#    return paths_organized, unique_paths
                              
 
 def find_simple(pointers,file_expt):
