@@ -37,10 +37,10 @@ def generate_subparsers(parser,epilog,project_drs):
     #Discover tree
     subparsers = parser.add_subparsers(help='Commands to discover available data on the archive',dest='command')
     discover(subparsers,epilog,project_drs)
-    list_paths(subparsers,epilog,project_drs)
+    list_fields(subparsers,epilog,project_drs)
     #slice(subparsers,epilog,project_drs)
     #simplify(subparsers,epilog,project_drs)
-    simulations(subparsers,epilog,project_drs)
+    #simulations(subparsers,epilog,project_drs)
 
     #Optimset tree
     optimset(subparsers,epilog,project_drs)
@@ -49,8 +49,7 @@ def generate_subparsers(parser,epilog,project_drs):
 
 def discover(subparsers,epilog,project_drs):
     #Find data
-    epilog_discover=textwrap.dedent(epilog+'\n\nThe output can be pretty printed by using:\n\
-                          cat out_diagnostic_headers_file | python -mjson.tool')
+    epilog_discover=epilog
     parser=subparsers.add_parser('discover',
                                            description=textwrap.dedent(
                                                 '''Returns pointers to models that have as a subset the requested experiments and variables.\n\
@@ -79,19 +78,22 @@ def discover(subparsers,epilog,project_drs):
                                         Listing separate fields is usually much quicker than the discovery step.')
     return parser
 
-def list_paths(subparsers,epilog,project_drs):
+def list_fields(subparsers,epilog,project_drs):
     #List_paths
-    parser=subparsers.add_parser('list_paths',
-                                           description=textwrap.dedent('List paths (on file system or url) to files containing:\n\
-                                                 ')
+    description=textwrap.dedent('List the unique combination of fields in the file.\n\
+    For example:\n\
+    1) to list the paths, use option \'-f path\'\n\
+    2) to list the simulations tuples use options \'-f institute -f model -f ensemble\'\n\
+         ').strip()
+    parser=subparsers.add_parser('list_fields',
+                                            description=description,
+                                            formatter_class=argparse.RawTextHelpFormatter,
+                                            epilog=epilog
                                            )
     slicing_arguments(parser,project_drs)
     input_arguments(parser)
-    parser.add_argument('--download',
-                         default='',
-                         type=str,
-                         help='Downloads the paths to the specified directory.\n\
-                               The user MUST have a valid certificate.')
+    parser.add_argument('-f','--field',action='append', type=str, choices=project_drs.base_drs,
+                                       help='List the field (or fields if repeated) found in the file' )
     return
 
 def slice(subparsers,epilog,project_drs):
@@ -127,15 +129,6 @@ def simplify(subparsers,epilog,project_drs):
 #    slicing_arguments(parser)
 #    return
 
-def simulations(subparsers,epilog,project_drs):
-    #Simulations
-    parser=subparsers.add_parser('simulations',
-                                   description=textwrap.dedent('Prints the (institute_model_ensemble) triples available in the pointers file.')
-                                   )
-    slicing_arguments(parser,project_drs)
-    input_arguments(parser)
-    return
-
 def optimset(subparsers,epilog,project_drs):
     #Find Optimset Months
     epilog_optimset=textwrap.dedent(epilog)
@@ -179,8 +172,11 @@ def remote_retrieve(subparsers,epilog):
                                            epilog=epilog_optimset,
                                          )
     input_arguments(parser)
-    parser.add_argument('out_data_netcdf_file',
-                                 help='Retrieved data as a netcdf file (output)')
+    parser.add_argument('out_destination',
+                             help='Destination directory for retrieval.')
+    parser.add_argument('--netcdf',default=False,action='store_true',
+                             help='If activated, the data is retrieved to a hierachical netCDF4 file.\n\
+                                   in this case, out_destination should point to a file.')
     return
 
 #def remote_retrieve(subparsers,epilog):
