@@ -11,6 +11,11 @@ import copy
 
 import os
 
+import cdb_query_archive_class
+import io_tools
+
+import retrieval_utils
+from itertools import groupby, count
 
 def md5_for_file(f, block_size=2**20):
     md5 = hashlib.md5()
@@ -185,14 +190,14 @@ def record_meta_data(header,output,paths_list,var,time_frequency,experiment):
 
 def record_paths(header,output,paths_list,var,time_frequency,experiment):
     #First order source files by preference:
-    sorts_list=['version','domain_id','file_type_id']
+    sorts_list=['version','file_type_id','domain_id']
     id_list=['domain','file_type','path','checksum','search']
     paths_ordering = order_paths_by_preference(paths_list,header,sorts_list,id_list)
     create_netcdf_pointers_file(header,output,paths_list,paths_ordering,id_list)
     return
 
 def retrieve_time_and_meta_data(header,output,source_files,var,experiment):
-    sorts_list=['version','domain_id','file_type_id']
+    sorts_list=['version','file_type_id','domain_id']
     id_list=['domain','file_type','path','checksum','search']
     paths_ordering = order_paths_by_preference(source_files,header,sorts_list,id_list)
 
@@ -218,7 +223,7 @@ def retrieve_time_and_meta_data(header,output,source_files,var,experiment):
         #This is important for piControl
         years_range=list(np.array(years_range)+np.min([date.year for date in time_axis_unique_date]))
     if 'months_list' in header.keys():
-        months_range=[int(month) for month in header['months_list'].split(',')]
+        months_range=header['months_list']
     else:
         months_range=range(1,13)
 
@@ -238,13 +243,13 @@ def retrieve_time_and_meta_data(header,output,source_files,var,experiment):
         paths_indices[path.replace('fileServer','dodsC')==table['paths']]=path_id
 
     #Remove paths that are not necessary over the requested time range:
-    paths_id_list=np.unique([np.min(paths_indices[time==time_axis])  for time_id, time in enumerate(time_axis_unique)])
+    paths_id_list=list(np.unique([np.min(paths_indices[time==time_axis])  for time_id, time in enumerate(time_axis_unique)]))
     useful_file_name_list=[path.split('/')[-1] for path in paths_ordering['path'][paths_id_list] ]
     for file_id, file in enumerate(paths_ordering):
         if not file_id in paths_id_list:
             file_name=paths_ordering['path'][file_id].split('/')[-1]
             if file_name in useful_file_name_list:
-                equivalent_file_id=paths_id_list[useful_file_name_list.index(filename)]
+                equivalent_file_id=paths_id_list[useful_file_name_list.index(file_name)]
                 if paths_ordering['checksum'][file_id]==paths_ordering['checksum'][equivalent_file_id]:
                     paths_id_list.append(file_id)
     paths_ordering=paths_ordering[np.sort(paths_id_list)]
@@ -413,7 +418,7 @@ def get_domain(path,file_type):
     if file_type=='HTTPServer':
         return '/'.join(path.split('/')[:3])
     elif file_type=='local_file':
-        return '/'.join(path.split('/')[:-1])
+        return '/'.join(path.split('/')[:2])
     else:
         return ''
 
@@ -443,12 +448,17 @@ def netcdf_calendar(data):
         calendar='standard'
     return calendar
 
-def retrieve_data(options):
-    data=netCDF4.Dataset(options.in_diagnostic_netcdf_file,'r')
-    output=netCDF4.Dataset(options.out_data_netcdf_file,'w')
-    output=descend_tree(options,data,output)
-    data.close()
-    output.close()
+def retrieve_data(options,project_drs):
+    if not options.netcdf:
+        paths_list=cdb_query_archive_class.SimpleTree(io_tools.open_netcdf(options,project_drs),options,project_drs).get_paths_list(options)
+        for path in paths_list:
+            retrieve_path(options,path)
+    else:
+        data=netCDF4.Dataset(options.in_diagnostic_netcdf_file,'r')
+        output=netCDF4.Dataset(options.out_destination,'w')
+        output=descend_tree(options,data,output)
+        data.close()
+        output.close()
     return
 
 def descend_tree(options,data,output):
@@ -467,33 +477,56 @@ def descend_tree_recursive(options,data,output):
                     output_grp.setncattr(att,getattr(data.groups[group],att))
             output_grp=descend_tree_recursive(options,data.groups[group],output_grp)
     else:
-        var_to_retrieve=[var for var in data.variables.keys() if 'indices' in data.variables[var].dimensions and var!='indices'][0]
-        if not 'time' in output.dimensions.keys():
-            create_time_axis(output,data,data.variables['time'][:])
+        if 'time' in data.dimensions.keys():
+            output=retrieve_remote_vars(options,data,output)
+        else:
+            #Fixed variables. Do not retrieve, just copy:
+            for var in data.variables.keys():
+                output=replicate_netcdf_var(output,data,var)
+                output.variables[var][:]=data.variables[var][:]
+            
+    return output
+            
+def retrieve_remote_vars(options,data,output):
+    if not 'time' in output.dimensions.keys():
+        create_time_axis(output,data,data.variables['time'][:])
+    vars_to_retrieve=[var for var in data.variables.keys() if 'indices' in data.variables[var].dimensions and var!='indices']
 
+    for var_to_retrieve in vars_to_retrieve:
         soft_link=np.ma.array(data.variables[var_to_retrieve][:]).data
+
+        #Sort the paths so that we query each only once:
+        sorting_paths=np.argsort(soft_link[:,0])
+        unique_paths_list_id=np.unique(soft_link[sorting_paths,0])
+        sorted_soft_link=soft_link[sorting_paths,:]
+
+        #Get list of paths:
         paths_list=data.variables['path'][:]
-        unique_paths_list_id=np.unique(soft_link[:,0])
+        
+        #Get attributes from variable to retrieve:
         remote_data=netCDF4.Dataset(paths_list[unique_paths_list_id[0]].replace('fileServer','dodsC'))
         output=replicate_netcdf_var(output,remote_data,var_to_retrieve,chunksize=-1)
         remote_data.close()
 
+        #Replicate all the other variables:
         for var in data.variables.keys():
             if not 'time' in data.variables[var].dimensions:
                 output=replicate_netcdf_var(output,data,var)
                 output.variables[var][:]=data.variables[var][:]
 
+        #Retrieve the data path by path. This is the most efficient method since paths only have to be queried once
         retrieved_data=map(retrieve_path_data,
                             zip(paths_list[unique_paths_list_id],
-                                [soft_link[soft_link[:,0]==path_id,1] for path_id in unique_paths_list_id],
+                                [sorted_soft_link[sorted_soft_link[:,0]==path_id,1] for path_id in unique_paths_list_id],
                                 [var_to_retrieve for path_id in unique_paths_list_id]
                                 )
                             )
-        #print np.concatenate(retrieved_data,axis=0).shape
-        output.variables[var_to_retrieve][:]=np.concatenate(retrieved_data,axis=0)
+
+        #print np.argsort(sorting_paths)
+        #Assign retrieved variables to output. We sort back the time axis:
+        output.variables[var_to_retrieve][:]=np.concatenate(retrieved_data,axis=0)[np.argsort(sorting_paths),...]
 
         output.sync()
-        
     return output
 
 def retrieve_path_data(in_tuple):
@@ -503,72 +536,57 @@ def retrieve_path_data(in_tuple):
 
     remote_data=netCDF4.Dataset(path)
     if len(indices)==1:
-        return np.reshape(remote_data.variables[var][indices,...],(1,)+remote_data.variables[var].shape[1:])
+        return np.reshape(grab_remote_indices(remote_data.variables[var],indices),(1,)+remote_data.variables[var].shape[1:])
     else:
-        return remote_data.variables[var][indices,...]
+        return grab_remote_indices(remote_data.variables[var],indices)
+
+def grab_remote_indices(variable,indices):
+    indices_sort=np.argsort(indices)
+    return np.concatenate(map(lambda x: variable[slice(*x),...],
+                                convert_indices_to_slices(indices[indices_sort])
+                             ),axis=0
+                             )[np.argsort(indices_sort),...]
+
+def convert_indices_to_slices(indices):
+    slices = []
+    for key, it in groupby(enumerate(indices), lambda x: x[1] - x[0]):
+        indices = [y for x, y in it]
+        if len(indices) == 1:
+            slices.append([indices[0]])
+        else:
+            slices.append([indices[0], indices[-1]+1])
+    return slices
+
+def retrieve_path(options,path):
+    decomposition=path[0].split('|')
+    if not (isinstance(decomposition,list) and len(decomposition)>1):
+        return
+
+    root_path=decomposition[0]
+    dest_name=options.out_destination+'/'.join(path[1:])+'/'+root_path.split('/')[-1]
+    try:
+        md5sum=md5_for_file(open(dest_name,'r'))
+    except:
+        md5sum=''
+    if md5sum==decomposition[1]:
+        print 'File '+dest_name+' found.'
+        print 'MD5 OK! Not retrieving.'
+        return
+
+    retrieval_utils.download_secure(root_path,dest_name)
+    try:
+        md5sum=md5_for_file(open(dest_name,'r'))
+    except:
+        md5sum=''
+    print 'Checking MD5 checksum of retrieved file...'
+    if md5sum!=decomposition[1]:
+        print('File '+dest_name+' does not have the same MD5 checksum as published on the ESGF. Removing this file...')
+        try:
+            os.remove(dest_name)
+        except:
+            pass
+    else:
+        print 'MD5 OK!'
+    return
     
 
-#READ NETCDF FILES
-#def descend_tree(project_drs,options,data,timestamp=None,tree=None):
-#    output_tuple=descend_tree_recursive(options,project_drs.base_drs,data,timestamp=timestamp,tree=tree)
-#    return output_tuple
-#
-#def descend_tree_recursive(options,tree_desc,data,timestamp=None,tree=None):
-#    if isinstance(tree_desc,list) and len(tree_desc)>0:
-#        local_tree_desc=tree_desc[0]
-#        next_tree_desc=tree_desc[1:]
-#        if ( local_tree_desc in dir(options) and 
-#            getattr(options,local_tree_desc) in data.groups.keys()):
-#            output=descend_tree_recursive(
-#                                  options,
-#                                  next_tree_desc,
-#                                  data.groups[getattr(options,local_tree_desc)],
-#                                  timestamp=timestamp,
-#                                  tree=tree)
-#    else:   
-#        if timestamp!=None:
-#            output=find_timestamp_index(options,tree_desc,data,timestamp)
-#        #elif tree!=None:
-#    return output
-#
-#def find_timestamp_index(options,tree_desc,data,timestamp):
-#    time_axis=data.variables['time'][:]
-#    timestamp_val=netCDF4.date2num(timestamp,units=data.variables['time'].units, calendar=netcdf_calendar(data))
-#    try:
-#        time_id=list(time_axis).index(timestamp_val)
-#    except:
-#        raise IOError('Timestamp not in file')
-#    temp=np.ma.array(data.variables[getattr(options,'var')][time_id,:]).data
-#    #Output the path from the first nonmasked element. The paths were sorted earlier
-#    #path_index=np.ma.nonzero(temp+1)[0]
-#    #time_index_in_path=temp[path_index]
-#    path_index=temp[0]
-#    time_index_in_path=temp[1]
-#    file_desc={}
-#    for id in ['path','version','checksum','domain']:
-#        file_desc[id]=data.variables[id][:]
-#
-#    #Next, we look for identical paths:
-#    domains_with_identical_files=[file_desc['domain'][i] for i in range(0,len(file_desc['path'])) if 
-#                                file_desc['path'][i].split('/')[-1] == file_desc['path'][path_index].split('/')[-1]
-#                            and file_desc['version'][i] == file_desc['version'][path_index]
-#                            and file_desc['checksum'][i] == file_desc['checksum'][path_index]
-#                            ]
-#    if options.domain in domains_with_identical_files:
-#        alternative_file=[file_desc['path'][i] for i in range(0,len(file_desc['path'])) if 
-#                                file_desc['path'][i].split('/')[-1] == file_desc['path'][path_index].split('/')[-1]
-#                            and file_desc['version'][i] == file_desc['version'][path_index]
-#                            and file_desc['checksum'][i] == file_desc['checksum'][path_index]
-#                            and file_desc['domain'][i] == options.domain
-#                            ]
-#        output={'path':alternative_file[0],
-#                'index':time_index_in_path,
-#                'timestamp':timestamp,
-#                'domains':domains_with_identical_files}
-#    else:
-#        #output_tuple=(file_desc['path'][path_index], time_index_in_path, timestamp, domains_with_identical_files)
-#        output={'path':file_desc['path'][path_index],
-#                'index':time_index_in_path,
-#                'timestamp':timestamp,
-#                'domains':domains_with_identical_files}
-#    return output
