@@ -102,15 +102,25 @@ def populate_pointers_from_netcdf_recursive(tree,infile):
             setattr(tree.file_expt,infile.groups[group].getncattr('level_name'),group)
             populate_pointers_from_netcdf_recursive(tree,infile.groups[group])
     else:
-        paths=infile.variables['path'][:]
-        for path_id, path in enumerate(paths):
+        if 'path' in infile.variables.keys():
+            paths=infile.variables['path'][:]
+            for path_id, path in enumerate(paths):
+                id_list=['file_type','search']
+                for id in id_list:
+                    setattr(tree.file_expt,id,infile.variables[id][path_id])
+                setattr(tree.file_expt,'path','|'.join([infile.variables['path'][path_id],
+                                                       infile.variables['checksum'][path_id]]))
+                setattr(tree.file_expt,'version','v'+str(infile.variables['version'][path_id]))
+                #print {att:getattr(tree.file_expt,att) for att in dir(tree.file_expt) if att[0]!='_'}
+                tree.add_item()
+        elif 'path' in infile.ncattrs():
+            #for fx variables:
             id_list=['file_type','search']
             for id in id_list:
-                setattr(tree.file_expt,id,infile.variables[id][path_id])
-            setattr(tree.file_expt,'path','|'.join([infile.variables['path'][path_id],
-                                                   infile.variables['checksum'][path_id]]))
-            setattr(tree.file_expt,'version','v'+str(infile.variables['version'][path_id]))
-            #print {att:getattr(tree.file_expt,att) for att in dir(tree.file_expt) if att[0]!='_'}
+                setattr(tree.file_expt,id,infile.getncattr(id))
+            setattr(tree.file_expt,'path','|'.join([infile.getncattr('path'),
+                                                   infile.getncattr('checksum')]))
+            setattr(tree.file_expt,'version',str(infile.getncattr('version')))
             tree.add_item()
     return
 
@@ -281,7 +291,8 @@ def replace_path(path_name,branch_desc):
 def create_database_from_tree(pointers,file_expt,tree,find_function):
     #Recursively creates a database from tree:
     if isinstance(tree,dict):
-        for value in [key for key in tree.keys() if key]:
+        #for value in [key for key in tree.keys() if key]:
+        for value in [key for key in tree.keys() if isinstance(key,basestring)]:
             if value[0] != '_':
                 file_expt_copy=copy.deepcopy(file_expt)
                 setattr(file_expt_copy,tree['_name'],value)
