@@ -1,5 +1,6 @@
 import copy
 from tree_utils import File_Expt
+from operator import itemgetter
 
 
 import sqlalchemy
@@ -64,24 +65,16 @@ def intersection(diagnostic,project_drs):
     for var_name in diagnostic.header['variable_list'].keys():
         variable_list_requested.append((var_name,)+tuple(diagnostic.header['variable_list'][var_name]))
 
-    model_list_combined=[model for model in model_list if model[:-1] in model_list_fx]
-    model_list_combined.extend([model + ('r0i0p0',) for model in model_list_fx])
+    #Step three: find the models to remove:
+    simulations_desc_indices_without_ensemble=range(0,len(project_drs.simulations_desc))
+    simulations_desc_indices_without_ensemble.remove(project_drs.simulations_desc.index('ensemble'))
+    model_list_combined=[model for model in model_list if itemgetter(*simulations_desc_indices_without_ensemble)(model) in model_list_fx]
+    models_to_remove=set(simulations_list).difference(model_list_combined)
 
-    #Delete the original tree to rebuild it with only the elements we want:
-    diagnostic.pointers.clear_tree(project_drs.discovered_drs)
-    for model in model_list_combined:
-            runs_list=diagnostic.pointers.session.query(
-                                      File_Expt
-                                     ).filter(sqlalchemy.and_(
-                                                                File_Expt.institute==model[0],
-                                                                File_Expt.model==model[1],
-                                                                File_Expt.ensemble==model[2],
-                                                             )).all()
-            for item in runs_list:
-                for val in dir(item):
-                    if val[0]!='_':
-                        setattr(diagnostic.pointers.file_expt,val,getattr(item,val))
-                diagnostic.pointers.add_item()
+    #Step four: remove from database:
+    for model in models_to_remove:
+        conditions=[ getattr(File_Expt,field)==model[field_id] for field_id, field in enumerate(itemgetter(*simulations_desc_indices_without_ensemble)(project_drs.simulations_desc))]
+        diagnostic.pointers.session.query(File_Expt).filter(*conditions).delete()
                 
     return
 
